@@ -25,6 +25,7 @@ use constant INSTALLER_BUILD
 use constant INSTALLER_MM   => 'CPANPLUS::Dist::MM';    
 use constant INSTALLER_SAMPLE   
                             => 'CPANPLUS::Dist::Sample';
+use constant INSTALLER_BASE => 'CPANPLUS::Dist::Base';                            
 
 use constant CONFIG         => 'CPANPLUS::Config';
 use constant CONFIG_USER    => 'CPANPLUS::Config::User';
@@ -241,7 +242,6 @@ use constant CALLING_FUNCTION
                             => sub { my $lvl = $_[0] || 0;
                                      return join '::', (caller(2+$lvl))[3] 
                                 };
-use constant DOT_EXISTS     => '.exists';     
 use constant PERL_CORE      => 'perl';
 
 use constant GET_XS_FILES   => sub { my $dir = $_[0] or return;
@@ -255,12 +255,41 @@ use constant GET_XS_FILES   => sub { my $dir = $_[0] or return;
                                      return @files;
                                 };  
 
-use constant ON_OLD_CYGWIN  => do { $^O eq 'cygwin' and $] < 5.008 
-                                    ? loc("Your perl version for %1 is too low; ".
-                                            "Require %2 or higher for this function",
-                                            $^O, '5.8.0' )
+use constant INSTALL_LOG_FILE 
+                            => sub { my $obj  = shift or return;
+                                     my $name = $obj->name; $name =~ s/::/-/g;
+                                     $name .= '-'. $obj->version;
+                                     $name .= '-'. scalar(time) . '.log';
+                                     return $name;
+                                };                                        
+
+use constant ON_WIN32       => $^O eq 'MSWin32';
+use constant ON_NETWARE     => $^O eq 'NetWare';
+use constant ON_CYGWIN      => $^O eq 'cygwin';
+use constant ON_VMS         => $^O eq 'VMS';
+
+use constant ON_OLD_CYGWIN  => do { ON_CYGWIN and $] < 5.008 
+                                    ? loc(
+                                       "Your perl version for %1 is too low; ".
+                                       "Require %2 or higher for this function",
+                                       $^O, '5.8.0' )
                                     : '';                                                                           
                                 };
+
+### XXX these 2 are probably obsolete -- check & remove;
+use constant DOT_EXISTS     => '.exists'; 
+
+use constant QUOTE_PERL_ONE_LINER 
+                            => sub { my $line = shift or return;
+
+                                     ### use double quotes on these systems
+                                     return qq["$line"] 
+                                        if ON_WIN32 || ON_NETWARE || ON_VMS;
+
+                                     ### single quotes on the rest
+                                     return qq['$line'];
+                            };   
+
 1;              
 
 # Local variables:
